import { isQuantitative } from '../../scale';
import { titlecase } from '../../util';
import { getStyleConfig } from '../common';
export function getAxisConfig(property, config, channel, orient, scaleType, style) {
    var _a, _b, _c;
    let styleConfig = getStyleConfig(property, style, config.style);
    if (styleConfig !== undefined) {
        return {
            configFrom: 'style',
            configValue: styleConfig
        };
    }
    const typeBasedConfigs = [
        ...(scaleType === 'band' ? ['axisBand', 'axisDiscrete'] : []),
        ...(scaleType === 'point' ? ['axisPoint', 'axisDiscrete'] : []),
        ...(isQuantitative(scaleType) ? ['axisQuantitative'] : []),
        ...(scaleType === 'time' || scaleType === 'utc' ? ['axisTemporal'] : [])
    ];
    const channelBasedConfig = channel === 'x' ? 'axisX' : 'axisY';
    // configTypes to loop, starting from higher precedence
    const axisConfigs = [
        ...typeBasedConfigs.map(c => channelBasedConfig + c.substr(4)),
        ...typeBasedConfigs,
        // X/Y
        channelBasedConfig,
        // axisTop, axisBottom, ...
        ...(orient ? ['axis' + titlecase(orient)] : []),
        'axis'
    ];
    // apply properties in config Types first
    for (const configType of axisConfigs) {
        if (((_a = config[configType]) === null || _a === void 0 ? void 0 : _a[property]) !== undefined) {
            return {
                configFrom: configType,
                configValue: config[configType][property]
            };
        }
    }
    // then apply style in config types
    for (const configType of axisConfigs) {
        if ((_b = config[configType]) === null || _b === void 0 ? void 0 : _b.style) {
            styleConfig = getStyleConfig(property, (_c = config[configType]) === null || _c === void 0 ? void 0 : _c.style, config.style);
            if (styleConfig !== undefined) {
                return {
                    configFrom: 'axis-config-style',
                    configValue: styleConfig
                };
            }
        }
    }
    return {};
}
//# sourceMappingURL=config.js.map